﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;

namespace RH.SharePointKonferenz
{
    /// <summary>
    /// Description of the toolpart. Override the GetToolParts method
    /// in your Web Part class to invoke this toolpart. To establish
    /// a reference to the Web Part the user has selected, use the
    /// ParentToolPane.SelectedWebPart property.
    /// </summary>
    public class CustomToolPart : ToolPart
    {
        private string _ErrorMessage = string.Empty;
        private string Property;

        // an event handler for the Init event
        private void CustomToolPartFooter_Init(object sender, EventArgs e)
        {
            Property = UniqueID + "Property";
        }

        /// <summary>
        /// Constructor for the class. A great place to set Set
        /// default values for additional base class properties
        /// here.
        /// <summary>
        public CustomToolPart()
        {
            // Set default properties
            Title = "by René Hézser";
            Init += CustomToolPartFooter_Init;
        }

        ///	<summary>
        ///	Called by the tool pane to apply property changes to
        /// the selected Web Part.
        ///	</summary>
        public override void ApplyChanges()
        {
            var webpart = (SharePointKonferenz)ParentToolPane.SelectedWebPart;
            webpart.Property = SPEncode.HtmlEncode(Page.Request.Form[Property]);
        }

        /// <summary>
        ///	If the ApplyChanges method succeeds, this method is
        /// called by the tool pane to refresh the specified
        /// property values in the toolpart user interface.
        /// </summary>
        public override void SyncChanges()
        {
        }

        /// <summary>
        ///	Called by the tool pane if the user discards changes
        /// to the selected Web Part.
        /// </summary>
        public override void CancelChanges()
        {
        }

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

            try
            {
                // Establish a reference to the Web Part.
                var webpart = (SharePointKonferenz)ParentToolPane.SelectedWebPart;
                var textBox = new TextBox {ID = "Property", Text = webpart.Property};
                Controls.Add(textBox);

                Controls.Add(new LiteralControl("<br/>"));

                var link = new HyperLink { NavigateUrl = "http://www.hezser.de/blog", Text = "www.hezser.de/blog", Target = "_blank" };
                Controls.Add(link);
            }
            catch (Exception ex)
            {
                _ErrorMessage += string.Format("Error in CreateChildControls: {0}", ex);
            }
        }

        /// <summary>
        /// Render this Tool part to the output parameter
        /// specified.
        /// </summary>
        /// <param name="writer">
        /// The HTML writer to write out to
        /// </param>
        protected override void RenderToolPart(HtmlTextWriter writer)
        {
            try
            {
                if (string.IsNullOrEmpty(_ErrorMessage) == false)
                {
                    writer.WriteLine(_ErrorMessage);
                }
                RenderChildren(writer);
            }
            catch (Exception ex)
            {
                writer.Write(string.Format("Error in CreateChildControls:\r\n{0}", Helper.GetExeption(ex)));
            }
        }
    }
}
