﻿using System;
using System.Linq;
using Microsoft.SharePoint;
using System.Web.UI;

namespace RH.SharePointKonferenz
{
    public static class Extensions
    {
        public static bool IsNullOrEmpty(this string s)
        {
            return string.IsNullOrEmpty(s);
        }

        public static bool GetList(this SPWeb web, string listTitle, out SPList list)
        {
            list = web.Lists.Cast<SPList>().Where(l => l.Title == listTitle).FirstOrDefault();
            return list != null;
        }

        public static bool FieldExists(this SPFieldCollection fields, string internalFieldName)
        {
            return fields.Cast<SPField>().Any(f => f.InternalName == internalFieldName);
        }

        /// <summary>
        /// Add a LiberalControls with &lt;br/&gt;
        /// </summary>
        /// <param name="control">the control, where the linebreak will be placed</param>
        /// <param name="rows">how many linebreaks</param>
        public static void AddLineBreak(this Control control, int rows)
        {
            for (int i = 0; i < rows; i++)
            {
                control.Controls.Add(new LiteralControl("</br>"));
            }
        }

        /// <summary>
        /// checks all fields if they are required, and if so, check if the item has a value for the field
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public static bool RequiredFieldsSet(this SPListItem item)
        {
            foreach (SPField field in item.Fields)
            {
                try
                {
                    if (field.Required == false) continue;

                    if (field.GetValidatedString(item[field.Id]).IsNullOrEmpty())
                    {
                        // field has no value
                        return false;
                    }
                }
                catch (FormatException)
                {
                    return false;
                }
            }

            return true;
        }
    }
}
