﻿using System;
using System.Linq;
using System.Runtime.InteropServices;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;
using Microsoft.SharePoint.Utilities;

namespace RH.SharePointKonferenz
{
    [Guid("32feed22-c890-4f9e-acd1-3285658cbbbc")]
    public class SharePointKonferenz : Microsoft.SharePoint.WebPartPages.WebPart
    {
        private string _Property;
        private SPList _SiteUsers;

        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [System.ComponentModel.Category("Property Group")]
        [WebDisplayName("Property")]
        [WebDescription("Meaningless Property")]
        public string Property
        {
            get
            {
                if (_Property == null)
                {
                    _Property = "Hello SharePoint";
                }
                return _Property;
            }
            set { _Property = value; }
        }

        public override ToolPart[] GetToolParts()
        {
            try
            {
                var toolparts = new ToolPart[2];
                var wptp = new WebPartToolPart();
                var custom = new CustomPropertyToolPart();
                toolparts[0] = custom;
                toolparts[1] = wptp;

                // This is the custom ToolPart.
                toolparts[0] = new CustomToolPart();

                return toolparts;
            }
            catch (Exception ex)
            {
                HandleException(new Exception(string.Format("Error in ToolPart: {0}", Helper.GetExeption(ex))));
            }
            return null;
        }

        /// <summary>
        /// Create all your controls here for rendering.
        /// Try to avoid using the RenderWebPart() method.
        /// </summary>
        protected override void CreateChildControls()
        {
            try
            {
                base.CreateChildControls();
                Helper.WriteULSVerbose("Entering CreateChildControls");

                CreateWelcomeText();
                this.AddLineBreak(1);

                Controls.Add(new LiteralControl(Property));
                this.AddLineBreak(2);

                RenderSiteUersAsHtml();

                GetSiteUsersList();
                GetUserForm();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (Parent.GetType().FullName == "Microsoft.SharePoint.WebPartPages.WebPartPreview")
            {
                Helper.WriteULSVerbose("WebPartPreview Mode");
                writer.Write("<img src=\"/_layouts/images/SharePointKonferenzFeature.gif\" border=\"0\" alt=\"SharePointKonferenz Webpart\">");
            }
            else
            {
                Helper.WriteULSVerbose("RenderContents");
                base.RenderContents(writer);
            }
        }

        /// <summary>
        /// Clear all child controls and add an error message for display.
        /// </summary>
        /// <param name="ex"></param>
        private void HandleException(Exception ex)
        {
            Controls.Clear();
            Controls.Add(new LiteralControl(Helper.GetExeption(ex)));
            Helper.WriteULSError(ex.ToString());
        }

        /// <summary>
        /// Get Site Users List, and render DefaultView
        /// </summary>
        private void RenderSiteUersAsHtml()
        {
            SPWeb web = SPContext.Current.Web;
            SPList list = web.Lists.Cast<SPList>().SingleOrDefault(l => l.Title == "User Information List");
            if (list == null)
            {
                var rootWeb = SPContext.Current.Site.RootWeb;
                list = rootWeb.Lists.Cast<SPList>().SingleOrDefault(l => l.Title == "User Information List");
            }
            SPView view = list.DefaultView;
            // update the query
            // view.Query = new Caml Query;

            Controls.Add(new LiteralControl(view.RenderAsHtml()));
        }

        /// <summary>
        /// get the Site Users List with elevated privileges
        /// </summary>
        private void GetSiteUsersList()
        {
            Helper.WriteULSVerbose("Connecting to Site Users Liste");

            using (var elevatedSite = new SPSite(SPContext.Current.Web.Url, SPContext.Current.Site.SystemAccount.UserToken))
            using (var elevatedWeb = elevatedSite.RootWeb)
            {
                if (elevatedWeb.GetList("User Information List", out _SiteUsers) == false)
                {
                    Helper.WriteULSError("Site Users Liste nicht gefunden");
                    throw new ArgumentException("Site Users Liste nicht gefunden.<br/>");
                }
            }
        }

        private void GetUserForm()
        {
            // only add the newForm, if the user may edit items
            if (_SiteUsers == null || !_SiteUsers.DoesUserHavePermissions(SPBasePermissions.EditListItems))
            {
                return;
            }

            // connect to the rootweb, to read the users list
            var rootWeb = SPContext.Current.Site.RootWeb;
            // get the current user and corresponding item in the user information list
            SPUser user = SPContext.Current.Web.CurrentUser;
            SPList list = rootWeb.Lists["User Information List"];
            SPListItem userItem = list.GetItemById(user.ID);

            // set the context, so the controls will work
            SPContext context = SPContext.GetContext(HttpContext.Current, userItem.ID, list.ID, rootWeb);

            // container for the controls
            var content = new HtmlGenericControl("div");
            content.AddLineBreak(1);
            content.Controls.Add(new Label
                                     {
                                         Text = "Edit current User",
                                         CssClass = "ms-sitetitle"
                                     });
            content.AddLineBreak(2);

            // Username
            content.Controls.Add(new Label {Text = "Username:"});
            content.AddLineBreak(1);
            var username = new TextField
                               {
                                   ID = "Username",
                                   FieldName = "Title",
                                   ItemId = userItem.ID,
                                   ListId = list.ID,
                                   ControlMode = SPControlMode.Edit,
                                   RenderContext = context,
                                   ItemContext = context
                               };
            content.Controls.Add(username);
            content.AddLineBreak(1);

            // about me
            content.Controls.Add(new Label {Text = "About me:"});
            content.AddLineBreak(1);
            var aboutMe = new RichTextField
                              {
                                  ID = "AboutMe",
                                  FieldName = "Notes",
                                  ItemId = userItem.ID,
                                  ListId = list.ID,
                                  ControlMode = SPControlMode.Edit,
                                  RenderContext = context,
                                  ItemContext = context
                              };
            content.Controls.Add(aboutMe);
            content.AddLineBreak(1);

            var newButton = new Button
                                {
                                    ID = "UpdateUserButton",
                                    Text = "Update User"
                                };
            newButton.Click += newButton_Click;
            content.Controls.Add(newButton);

            Controls.Add(content);
        }

        void newButton_Click(object sender, EventArgs e)
        {
            try
            {
                var rootWeb = SPContext.Current.Site.RootWeb;
                // get the current user and corresponding item in the user information list
                SPUser user = SPContext.Current.Web.CurrentUser;
                SPList list = rootWeb.Lists["User Information List"];
                //SPListItem userItem = list.GetItemById(user.ID);
                // get value from controls
                var usernameField = (TextField) FindControl("Username");
                var aboutMeField = (NoteField) FindControl("AboutMe");

                var item = list.GetItemById(user.ID);
                item[SPBuiltInFieldId.Title] = usernameField.Value;
                item[SPBuiltInFieldId.Notes] = aboutMeField.Value;
                if (item.RequiredFieldsSet())
                {
                    item.Update();
                    // after the item has been saved, it will not be updated in the previously added control
                    // the page needs to be reloaded
                    return;
                }
                Controls.Add(new Label
                                 {
                                     Text = "Bitte alle Felder ausfüllen",
                                     CssClass = "ms-error"
                                 });
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        /// <summary>
        /// Show localized Welcome text
        /// </summary>
        private void CreateWelcomeText()
        {
            uint lcid = SPContext.Current.Web.RegionalSettings.LocaleId;
            string welcome = SPUtility.GetLocalizedString("$Resources:Welcome", "RHSharePointKonferenz", lcid);

            if (SPContext.Current.Web.CurrentUser != null)
            {
                welcome += " " + SPContext.Current.Web.CurrentUser.Name;
            }

            Controls.Add(new LiteralControl(welcome));
            this.AddLineBreak(1);
        }
    }
}
